/*************************************************************************
 *   Copyright (C) 2011-2017 by Paul-Louis Ageneau                       *
 *   paul-louis (at) ageneau (dot) org                                   *
 *                                                                       *
 *   This file is part of Teapotnet.                                     *
 *                                                                       *
 *   TeapotNet is free software: you can redistribute it and/or modify   *
 *   it under the terms of the GNU Affero General Public License as      *
 *   published by the Free Software Foundation, either version 3 of      *
 *   the License, or (at your option) any later version.                 *
 *                                                                       *
 *   TeapotNet is distributed in the hope that it will be useful, but    *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of          *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the        *
 *   GNU Affero General Public License for more details.                 *
 *                                                                       *
 *   You should have received a copy of the GNU Affero General Public    *
 *   License along with TeapotNet.                                       *
 *   If not, see <http://www.gnu.org/licenses/>.                         *
 *************************************************************************/

function setMailReceiverRec(url, object, period, next) {
	if(typeof this.messagesTimeout != 'undefined')
		clearTimeout(this.messagesTimeout);

	$.ajax({
		url: url + (url.contains('?') ? '&' : '?') + 'next=' + next + '&timeout=0',
		dataType: 'json',
		timeout: 300000
	})
	.done(function(array) {
		var user = getAuthenticatedUser();
		var userIdentifier = getAuthenticatedUserIdentifier();
		var posturl = url;
		var count = 0;
		$.each(array, function(i, mail) {
			next++;
			$(object).find('p').remove();

			var id = "message_" + mail.digest;
			var idReply = "reply_" + mail.digest;
			var idParent = (mail.parent ? "message_" + mail.parent : "");
			var idParentReply = (mail.parent ? "reply_" + mail.parent : "");
			var author = (mail.author ? mail.author : "Anonymous");

			if(idParent && $('#'+idParent).length == 0) return;	// Parent not found
			$('#'+id).remove();

			var div = '<div id="'+id+'" class="message"><span class="header"><span class="author">'+author+'</span><span class="date">'+formatTime(mail.time).escape()+'</span><span class="time" style="display:none">'+mail.time+'</span></span><span class="content"></span></div>';

			if(!idParent) {
				$(object).prepend('<div class="conversation">'+div+'</div>');
				$('#'+id).append('<div class="buttonsbar"></div>');

				if(userIdentifier && mail.identifier != userIdentifier && typeof TokenMail != 'undefined') {
					$('#'+id+' .buttonsbar').append('<a href="#" class="button passlink"><img alt="Pass" src="/static/arrow_pass.png"></a>');
					(function(id, digest) {
						$('#'+id+' .passlink').click(function() {
							if(confirm('Do you want to pass this message to your contacts ?')) {
								$.post(posturl, {action: "pass", digest: digest, token: TokenMail})
								.done(function(data) {
									$('#'+id+' .passlink').replaceWith('<span class="button"><img alt="Passed" src="/static/arrow_passed.png"></span>');
								});
							}
							return false;
						});
						$.ajax({
							url: '/user/'+user+'/board/?json&digest='+digest,
							dataType: 'json',
							timeout: 10000
						})
						.done(function(contact) {
							$('#'+id+' .passlink').replaceWith('<span class="button"><img alt="Passed" src="/static/arrow_passed.png"></span>');
						});
					})(id, mail.digest);
				}

				$('#'+id+' .buttonsbar').prepend('<a href="#" class="button replylink"><img alt="Reply" src="/static/arrow_reply.png"></a>');
				(function(idReply) {
					$('#'+id+' .replylink').click(function() {
						$('#'+idReply).toggle();
						$('#'+idReply+' textarea').focus();
						return false;
					});
				})(idReply);
			}
			else {
				$('#'+idParentReply).before(div);	// insert before parent reply
				$('#'+id).addClass('childmessage');
			}

			if(user && 'identifier' in mail) {
				(function(id, identifier) {
					if(identifier == userIdentifier) {
						$('#'+id+' .author').html('<a href="/user/'+user+'/myself/">'+user+'</a>');
					}
					else {
						var url = '/user/'+user+'/contacts/';
						$.ajax({
							url: url+'?json&id='+identifier,
							dataType: 'json',
							timeout: 10000,
							error: function (xhr) {
								if(xhr.status == 404 && typeof TokenContact != 'undefined') {
									$('#'+id+' .author').append('&nbsp;<form name="addform'+id+'" action="'+url+'" method="post" enctype="application/x-www-form-urlencoded"><input type="hidden" name="token" value="'+TokenContact+'"><input type="hidden" name="action" value="add"><input type="hidden" name="id" value="'+identifier+'"><input type="hidden" name="name" value="'+$('#'+id+' .author').text()+'"></form><a href="#" class="addlink"><img src="/static/add.png" alt="+"></a>');
									$('#'+id+' .author .addlink').click(function() {
										$(this).hide();
										$('#'+id+' .author form').submit();
										return false;
									});
								}
							}
						})
						.done(function(contact) {
							$('#'+id+' .author').html('<a href="'+contact.prefix+'/">'+contact.name+'</a>');
						});
					}
				})(id, mail.identifier);
			}

			// Reply form
			$('#'+id).parent().append('<div id='+idReply+' class="reply"><div class="replypanel"><form name="replyform'+id+'" action="'+posturl+'" method="post" enctype="application/x-www-form-urlencoded"><textarea class="replyinput" name="message"></textarea><input type="hidden" name="attachment"><input type="hidden" name="attachmentname"><input type="hidden" name="parent" value="'+mail.digest+'"><input type="hidden" name="public" value="1"></form></div><div class="attachedfile"></div><div class="fileselector"></div></div>');
			$('#'+idReply).hide();
			$('#'+idReply+' .attachedfile').hide();
			$('#'+idReply+' form').ajaxForm(function() {
				$('#'+idReply+' form').resetForm();
				$('#'+idReply+' .attachedfile').html('');
				$('#'+idReply+' .fileselector').html('');
				$('#'+idReply).hide();
			});

			if(typeof UrlSelector != 'undefined' && typeof TokenDirectory != 'undefined') {
				$('#'+idReply+' .replypanel').prepend('<a class="button" href="#"><img alt="File" src="/static/paperclip.png"></a>');

				(function(idReply) {
					$('#'+idReply+' .button').click(function() {
						createFileSelector(UrlSelector, '#'+idReply+' .fileselector', '#'+idReply+' input[name="attachment"]', '#'+idReply+' input[name="attachmentname"]', TokenDirectory);
						return false;
					});
					$('#'+idReply+' input[name="attachment"]').change(function() {
						$('#'+idReply+' .attachedfile').html('').hide();
						var filename = $('#'+idReply+' input[name="attachmentname"]').val();
						if(filename != '') {
							$('#'+idReply+' .attachedfile')
								.append('<img class=\"icon\" src=\"/static/file.png\">')
								.append('<span class=\"filename\">'+filename+'</span>')
								.show();
						}
						var input = $('#'+idReply+' .replyinput');
						input.focus();
						if(input.val() == '') {
							input.val(filename).select();
						}
					});
				})(idReply);
			}

			$('#'+id+' .content').html(mail.content.escape().linkify().split("\n").join("<br>"));

			if(mail.attachments && mail.attachments[0]) {
				$('#'+id+' .header').after('<span class="attachment"></span>');
				$('#'+id+' .attachment').html('<img class="icon" src="/static/smallpaperclip.png">Loading attachment...');

				var url = '/file/'+mail.attachments[0];	// TODO
				(function(id, url) {
					var request = $.ajax({
						type: "HEAD",
						url: url,
						timeout: 60000
					})
					.done(function(data) {
						var name = request.getResponseHeader('Content-Name');
						var type = request.getResponseHeader('Content-Type');
						var media = type.substr(0, type.indexOf('/'));

						var content = '';
						if(media == 'image') {
							content = '<a href="'+url+'" target="_blank"><img class="preview" src="'+url+'" alt="'+name.escape()+'"></a><img class="clip" src="/static/clip.png">';
						}
						else if(media == 'audio' || media == 'video') {
							var usePlaylist = (deviceAgent.indexOf('android') < 0);
							content = '<span class="filename"><a href="'+url+'?download=1"><img class="icon" src="/static/down.png"></a><a href="'+url+(usePlaylist ? '?play=1' : '')+'"><img class="icon" src="/static/file.png">'+name.escape()+'</a></span><img class="clip" src="/static/clip.png">';
						}
						else {
							content = '<span class="filename"><a href="'+url+'" target="_blank"><img class="icon" src="/static/file.png">'+name.escape()+'</a></span><img class="clip" src="/static/clip.png">';
						}

						transition('#'+id+' .attachment', '<span class="attached">'+content+'</a>');

						setTimeout(function() {
							$(object).scrollTop($(object)[0].scrollHeight);
						}, 10);
					})
					.fail(function(jqXHR, textStatus) {
						$('#'+id+' .attachment').html('<img class="icon" src="/static/paperclip.png">Attachment not available');
					});

				})(id, url);
			}

			$('#'+id).append('<span class="footer"></span>');

			count++;
		});

		// Order messages
		if(count > 0) {
			$(object).html($(object).children().detach().sort(function(a,b){
				return $(a).find(".time").text() < $(b).find(".time").text();
			}));

			if(isPageHidden()) {
				NbNewMessages+= count;
				top.document.title = '(' + NbNewMessages + ') ' + BaseTitle;
				playMailSound();
			}
		}

		this.messagesTimeout = setTimeout(function() {
			setMailReceiverRec(url, object, period, next);
		}, period);
	})
	.fail(function(jqXHR, textStatus) {
		this.messagesTimeout = setTimeout(function() {
			setMailReceiverRec(url, object, period, next);
		}, period);
	});
}

function setMailReceiver(url, object, period, next) {
	setMailReceiverRec(url, object, period, 0);
}
